package CCM::CCM;
use Exporter;
our @ISA = ('Exporter');
our @EXPORT = qw(extract_seed_page);

use strict;
use warnings;
#use diagnostics;
use HTML::TreeBuilder;

use CCM::Common;
use CCM::Util;
use CCM::Debug;
use CCM::PerlUtil;

=head1 NAME

CCM

=head1 SYNOPSIS

extract a seed page based on CCM algorithm

=head1 DESCRIPTION

Using CCM algorithm to get region info (the data region node and two neighboring nodes immediately under data region node). With region info, boundary tags will be extracted. Data record will extracted using boundary tag information.

=cut

sub get_region_info (\@\%$$$);

sub extract_seed_page($$$\@\%){
  my ($filepath, $keyword, $percent, $phrase_list_ref, $ranked_phrase_list_ref) = @_;
  my $tree = get_html_tree($filepath);
  my @phrase_list = @$phrase_list_ref;
  if ($#phrase_list < 1) {
    print "Cannot extract page that has less than two occurrances of keywords: $keyword.\n";
    return;
  }
  my %phrase_hash = get_phrase_hash(@phrase_list);
  my $min_count = ($#phrase_list + 1) * $percent;
  my %ranked_phrase_list = %$ranked_phrase_list_ref;
  #trace("all phrase list...");
  #map {print $_ . "\n"} @phrase_list;
  #trace("\nranked phrase list...");
  #map {trace("phrase: " . $_ . "\n")} (keys %ranked_phrase_list);

  my @local_ranked_phrase_list = sort {$ranked_phrase_list{$b} <=> $ranked_phrase_list{$a}} @phrase_list;
  #trace("local_ranked_phrase_list: $#local_ranked_phrase_list");
  #print "phrase_list count: " . $#phrase_list . "\n";
  my %boundary_info_list = ();
  my %unmatched_phrase_hash = %phrase_hash;
  my @record_list;
  my @unmatched_phrases = @phrase_list;
  my $last_count = -1; # to prevent an infinite loop
  while ((scalar @unmatched_phrases) >= $min_count 
	 && ($last_count != $#unmatched_phrases)) {
    #trace("phrase_list: $#phrase_list");
    #trace("unmatched phrases: $#unmatched_phrases");
    $last_count = $#unmatched_phrases;
    my @region_info 
      = get_region_info(@phrase_list, 
			%ranked_phrase_list, 
			$tree, $local_ranked_phrase_list[0], $keyword);
    my ($records_ref, $boundary_info_ref, $unmatched_phrase_hash_ref) = 
      extract_using_region_info(@region_info, 
				%unmatched_phrase_hash, $keyword);
    my @records = @$records_ref;
    trace("record count: $#records");
    #my $min_record_count = $min_count * 0.5;
    my $min_record_count = 1;
    $min_record_count = ($min_record_count > 1) ? $min_record_count : 1;
    if ($#records > $min_record_count) { #if not extracting enough, don't count
      push @record_list, $records_ref;
    } else {
    #Should record those single matched results
    }
    my $region = $region_info[0];
    $boundary_info_list{$region->address} = $boundary_info_ref;
    %unmatched_phrase_hash = %$unmatched_phrase_hash_ref;
    detach_records(@records);
    #$region->detach(); #Remove this region and start over.
    my @node_list = get_phrase_node_list($tree, $keyword);
    #trace("Node list count: " . $#node_list);
    @phrase_list = get_phrase_list_from_nodes(@node_list, $keyword);

    @unmatched_phrases = @phrase_list;
    %phrase_hash = get_phrase_hash(@phrase_list);
    #foreach my $p (@phrase_list) {
    #  print $p . "\n" unless (defined($ranked_phrase_list{$p}));
    #}
    @local_ranked_phrase_list = sort {$ranked_phrase_list{$b} <=> $ranked_phrase_list{$a}} @phrase_list;
  }
  return ($tree, \@record_list, \%boundary_info_list, \@unmatched_phrases);
}

#TODO: it's better to transform it to all use node information.
sub get_neighboring_seeds (\@\%$$$){
#input: * array of phrase list in the order of appearance
#       * overall ranked phrase list;
#       * seed
#       * tree
#output: * seed_node (always the left one)
#        * neighbor_seed_node
  my ($phrase_list_ref, $ranked_phrase_list_ref, $seed, $tree, $keyword) = @_;
  my @phrase_list = @$phrase_list_ref;
  my %ranked_phrase_list = %$ranked_phrase_list_ref;
  #trace("get_neighboring_seeds");
  #trace("seed: $seed\n");
  my $seed_node = get_phrase_node($tree, $seed);;
  my $neighbor_seed;
  my $neighbor_seed_node;
  my $switch_seed = 0; # we'll make the left one to be the seed
  for my $i (0 .. $#phrase_list) {
    if ($seed eq $phrase_list[$i]) {
      if ($i == 0) {
	$neighbor_seed = $phrase_list[$i+1];
	#trace("seed_node 0: " . $seed_node->address);
	$neighbor_seed_node = 
	  get_neighboring_phrase_node($tree, $neighbor_seed, $seed_node);
      } elsif ($i == $#phrase_list) {
	$neighbor_seed = $phrase_list[$i-1];
	#trace("seed_node last: " . $seed_node->address);
	$neighbor_seed_node = 
	  get_neighboring_phrase_node($tree, $neighbor_seed, $seed_node);
	$switch_seed = 1;
      } else {
	
	my $candidate1_node = 
	  get_neighboring_phrase_node($tree, $phrase_list[$i-1], $seed_node);
	my $candidate2_node = 
	  get_neighboring_phrase_node($tree, $phrase_list[$i+1], $seed_node);
	my $ancestor1 = get_common_ancestor($seed_node, $candidate1_node);
	my $ancestor2 = get_common_ancestor($seed_node, $candidate2_node);
	my $count1 = get_phrase_node_count($ancestor1, $keyword);
	my $count2 = get_phrase_node_count($ancestor2, $keyword);
	if ($count1 < $count2) {
	  $neighbor_seed = $phrase_list[$i-1];
	  $neighbor_seed_node = $candidate1_node;
	} elsif ($count2 < $count1) {
	  $neighbor_seed = $phrase_list[$i+1];
	  $neighbor_seed_node = $candidate2_node;
	} else {
	  if (($ranked_phrase_list{$phrase_list[$i-1]} >= 
	   $ranked_phrase_list{$phrase_list[$i+1]})) {
	    $neighbor_seed_node = $candidate1_node;
	    $neighbor_seed = $phrase_list[$i-1]
	  } else {
	    $neighbor_seed_node = $candidate2_node;
	    $neighbor_seed = $phrase_list[$i+1];
	  }
      }
	if ($neighbor_seed eq $phrase_list[$i-1]) {
	  $switch_seed = 1;
	}
      }
      last;
    }
  }
  if ($switch_seed) {
    my $temp = $seed_node;
    $seed_node = $neighbor_seed_node;
    $neighbor_seed_node = $temp;
  }
  #trace("seed: " . $seed_node->as_text);
  #trace("neighboring_seed: " . $neighbor_seed_node->as_text);
  return ($seed_node, $neighbor_seed_node);
}

sub get_region_info (\@\%$$$){
#output: ($region_node, n0, n1, @lineage_list)
#        @seed_lineage is the tag list from leaf phrase node up to region node 

 my ($phrase_list_ref, $ranked_phrase_list_ref, $tree, $no1, $keyword) = @_;
  my %ranked_phrase_list = %$ranked_phrase_list_ref;
  my @phrase_list = @$phrase_list_ref;

  my ($seed_node, $neighbor_node) = get_neighboring_seeds(@phrase_list, %ranked_phrase_list, $no1, $tree, $keyword);
 #trace ("seed: " .  $seed . "\n");
 #trace ("neighbor_seed: " . $neighbor_seed . "\n");

 my $ancestor = get_common_ancestor($seed_node, $neighbor_node);
 my $ancestor_address = $ancestor->address;
 #trace($ancestor_address);
 $seed_node->address =~ /^($ancestor_address\.\d+)/;
 #trace($1);
 my $n0_level1 = $tree->address($1);
 trace($n0_level1->address);
 #trace("n0: " . $n0_level1->as_text);
 $neighbor_node->address =~ /^($ancestor_address\.\d+)/;
 my $n1_level1 = $tree->address($1);
 trace($n1_level1->address);
 #trace("n1: " . $n1_level1->as_text);
 my @seed_lineage = get_seed_lineage($seed_node, $n0_level1);
 return ($n0_level1->parent, $n0_level1, $n1_level1, \@seed_lineage);
}
