package CCM::Debug;
use Exporter;
our @ISA = ('Exporter');
our @EXPORT = qw(output_records_list output_records output_element 
		 output_unmatched_list trace_node_list trace
		 trace_child_tags);

use strict;
use warnings;
use diagnostics;
use HTML::TreeBuilder;
use Tie::RefHash; #don't use as function return.

=head1 NAME

Debug

=head1 SYNOPSIS

Debug utility functions

=head1 DESCRIPTION

Output and debug related functions.

=cut

sub output_records_list{
  my @records_list = @_;
  my $list_num = 1;
  foreach my $records_ref(@records_list) {
    print "record list: " . $list_num++ . "\n";
    my @records = @$records_ref;
    output_records(@records);
  }
}
sub output_records {
  my @record_list = @_;
  my $record_count = 1;
  for my $each_records(@record_list) {
    my @records = @$each_records;
    my @texts = map {output_element($_)} @records;
    print "record: " . $record_count++ . "\n";
    map {print $_ . "\n"} @texts;
    print "\n";
  }
}

sub trace_child_tags{
  my $parent = shift;
  my @lst = $parent->content_list;
  #$parent->dump;
  print "child tags for " . $parent->address . " " . $parent->tag . "\n";
  map {print $_->tag . " " if (ref $_ eq "HTML::Element")} @lst;
}
sub output_element{
  my $ele = shift;
  if (defined($ele)) {
    my @text_nodes = $ele->look_down(sub {my @descendants = $_[0]->descendants; return ($#descendants < 0 && (!$_[0]->is_empty()));}); #all leaf
    my @texts = map {$_->as_text()} @text_nodes;
    #map {print $_ . "\n"} @texts;
    my $rtn = join " ", @texts;
    #print $rtn . "\n";
    return $rtn;
  }
}

sub output_unmatched_list {
  my @unmatched_list = @_;
  print "start unmatched list...\n";
  foreach my $phrase (@unmatched_list) {
    print "  $phrase\n";
  }
  print "end unmatched list.\n";
}


sub trace_node_list{
  my @node_list = @_;
  foreach my $node(@node_list) {
    trace($node->as_HTML);
  }
}
sub trace {
  my $msg = shift;
  #print "$msg\n";
}

sub output_node_list {
  map {trace($_->tag);} @_;
}
