
package CCM::SCM;
use Exporter;
our @ISA = ('Exporter');
our @EXPORT = qw(extract_seed_page);

use strict;
use warnings;
use diagnostics;
use HTML::TreeBuilder;
use Tie::RefHash; #don't use as function return.

use CCM::Common;
use CCM::Util;
use CCM::Debug;

=head1 NAME

SCM

=head1 SYNOPSIS

An alternative algorithm to CCM that works on singe page.

=head1 DESCRIPTION

An alternative that does not rely on multiple sites
in absence of a meaningful ranked list.
Also is a good indicator about the usefulness of multiple sites
Basically we grow all the phrase until we find a common ancestor that
parents x% of all the phrase nodes. Those get extracted and we keep grow 
 others until we have less that x% left.

=cut

#Cannot use reference has hash key unless you tie it, 
# but that'll also tied up all caller, so choose to use address string mostly.
sub grow_node_list(\@$) {
  #input: *phrase node list
  #       *percent
  #output: * array of array: list of (record leaf node list)
  #        * array of unmatched node
  my ($node_list_ref, $percent) = @_;
  my @node_list = @$node_list_ref;
  
  my $total = $#node_list + 1;
  my $matched_count = $total * $percent;

  #this is the main list to be processed so that we can keep track of all the 
  #leaf node address.
  my %address_hash = (); #($leaf_address -> current node address)
  my %candidates_list = (); #($region_str -> ($region_node, @leaf_nodes))
  my @unmatched_list = ();
  my $root = $node_list[0]->root;
  foreach my $node (@node_list) {
    if (defined($node)) {
      $address_hash{$node->address} = $node->address;
    }
  }

  while ((scalar keys %address_hash) > 0) {
    my %match_hash = ();	#parent_address -> leaf address
    while (my ($leaf_address, $node_address) = each(%address_hash)) {
      unless ($node_address ne "") {
	push @unmatched_list, $leaf_address;
	next;
      }
      unless (exists($match_hash{$node_address})) {
	$match_hash{$node_address} = [($leaf_address)];
	#trace("parent tag: " . $parent->tag());
      } else {
	push @{$match_hash{$node_address}}, $leaf_address;
      }
    }
    my %next_address_hash = ();
    while (my ($ancestor_address, $addresses_ref) = each(%match_hash)) {
      unless ($ancestor_address ne "") {
	push @unmatched_list, $addresses_ref;
	next;
      }
      #trace("ancestor address: " . $ancestor_address);
      my @addresses = @$addresses_ref;
      #trace("match nodes count: $#addresses");
      if ($#addresses > $matched_count) {
	$candidates_list{$ancestor_address} = [@addresses];
      } elsif ($#addresses == 0) {   #i
	$next_address_hash{$addresses[0]} = $ancestor_address;
	#trace($key->tag);
      } else {
	foreach my $unmatched (@addresses) {
	  push @unmatched_list, $unmatched;
	}			#otherwise is premature converge and we don't consider them
      }
    }				#while %match_hash

    %address_hash = %next_address_hash;

    while (my ($leaf_addr, $ancestor_addr) = each(%address_hash)) {
      my $node = $root->address($ancestor_addr);
      if (defined($node) && defined($node->parent)) {
	$address_hash{$leaf_addr} = $node->parent->address;
      } else {
	$address_hash{$leaf_addr} = "";
      }
    }
  } #outmost while
=trace
  while(my ($region_address, $leaf_addresses_ref) = each(%candidates_list)) {
    trace($region_address);
    map {trace($_);} @$leaf_addresses_ref;
  }
=cut
  return (\%candidates_list, \@unmatched_list);
}

#for single page processing, we simply pick two as seed and neighbor
sub get_region_info_list($\%){
#output *  ($region_node, $n0, n1)
  my ($root, $candidates_list_ref) = @_;
=trace
  trace("extract_seed_page");
  while(my ($region_address, $leaf_addresses_ref) = each(%$candidates_list_ref)) {
    trace($region_address);
    map {trace($_);} @$leaf_addresses_ref;
  }
=cut
  my @rtn_list = ();
  my %candidates_list = %$candidates_list_ref;
  while(my ($region_address, $leaf_addresses_ref) = each(%candidates_list)) {
    #trace($region_address);
    my $region_node = $root->address($region_address);
    my @leaf_addresses = @$leaf_addresses_ref;
    next if ($#leaf_addresses < 1);
    my %level1_hash = ();
    my @nodes = map {$region_node->address($_)} @leaf_addresses;
    #for my $node(@nodes) {
    #  my $level1_node = get_level1_node($node, $region_node);
    #}
    my @level1_nodes = map {get_level1_node($_, $region_node)} @nodes;
    #find the first two
    my $level1_hash = ();
    foreach my $level1_node(@level1_nodes) {
      $level1_hash{$level1_node->address} = $level1_node;
    }
    my @seed_lineage = get_seed_lineage($nodes[0], $level1_nodes[0]);
    my @level1s = $region_node->content_list();
    my $left = $level1s[0];
    my @n0_n1 = ();
    my $count = 0;
    while (defined($left) && $count < 2) {
      if (exists($level1_hash{$left->address})) {
	push @n0_n1, $level1_hash{$left->address};
	++$count;
      }
      $left = $left->right;
    }
    #trace($region_node->tag);
    #trace($n0_n1[0]->as_HTML);
    #trace($n0_n1[1]->as_HTML);
    my @lst = ($region_node, $n0_n1[0], $n0_n1[1],\@seed_lineage);
    #trace($lst[1][0]->as_HTML);
    push @rtn_list, [@lst];
  }
  return @rtn_list;
}

sub extract_seed_page($$$\@){
  my ($filepath, $keyword, $percent, $phrase_list_ref) = @_;
  my $tree = get_html_tree($filepath);
  my @node_list = get_phrase_node_list($tree, $keyword);
  #output_node_list(@node_list);
  my ($seeds_list_ref, $unmatched_list_ref) = grow_node_list(@node_list, $percent);
=trace
  trace("extract_seed_page");
  while(my ($region_address, $leaf_addresses_ref) = each(%$seeds_list_ref)) {
    trace($region_address);
    map {trace($_);} @$leaf_addresses_ref;
  }
=cut
  my @unmatched_list = @$unmatched_list_ref;
  my @phrase_list = @$phrase_list_ref;
  my %phrase_hash = get_phrase_hash(@phrase_list);
  my @region_info_list = get_region_info_list($tree, %$seeds_list_ref);
=trace
  foreach my $region_info_ref(@region_info_list) {
    my @lst = @$region_info_ref;
    trace("  " . $lst[0]->as_HTML);
    trace("  " . $lst[1]->as_HTML);
    trace("  " . $lst[2]->as_HTML);
  }
=cut
  my @extract_results = 
    extract_using_region_info_list(@region_info_list, %phrase_hash, $keyword);
  #$tree->delete; #cannot delete here
  return ($tree, @extract_results);
}
